unit SalutUnit;

{ SalutUnit (C) by Vladimir Kladov, 2005

  KOL object (TSalut) to produce funny graphic and sound output
  very like to salut effect. See salut_sample project.

\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\\
mailto:   bonanzas@online.sinor.ru
Web-page: http://bonanzas.rinet.ru
}

interface

uses Windows, KOL, kolmath, MMSystem, err;

{$DEFINE USEDIRECTSND}
{$O+}

type
  TSoundEffect = ( seRocket, seExplode, seShort );

  PSalut = ^TSalut;
  TSalut = object( TObj )
  protected
    TmpBmp: PBitmap;
    TmpFrameIdx: Integer;
    Rockets: PList;
    Flares: PList;
    TimesNoRockets: Integer;
    fCritSec: TRTLCriticalSection;
    procedure Init; virtual;
  public
    FrameIdx: Integer;
    Seed: Integer;
    Width, Height: Integer;
    Frequency: Integer;
    ValleyCount: Integer;
    OnEndSalut: TOnEvent;
    procedure Frame;
    procedure Draw2Bmp( Bmp, Mask: PBitmap );
    procedure Draw2DC( DC: HDC; x0, y0: Integer );
    destructor Destroy; virtual;
  public
    function CreateRocket: PObj;
    function BoundsRect: TRect;
  public
    ShortTimer: PTimer;
    ShortTimerCount: Integer;
    procedure LoadSounds( var List: PList; const Path: String );
    procedure Sound( Effect: TSoundEffect; Volume,Pan: Integer );
    procedure ShortTimerTick( Sender: PObj );
  end;

function NewSalut( Awidth, Aheight, Afrequency, Acount: Integer ): PSalut;

implementation

{$IFDEF USEDIRECTSND}
  uses MultiWav;
{$ENDIF}

var
  RocketSounds, ExplodeSounds, ShortSounds: PList;

const
  MaxBright = 10;
  Colors: array[ 0..6 ] of TColor = ( clRed, clLime, clYellow, clAqua, clFuchsia,
    clBlue, clWhite );

type
  PFlare = ^TFlare;
  TFlare = object( TObj )
    Salut: PSalut;
    Color: TColor;
    Bright: Integer;
    StartPos: TPoint;
    Position: TPoint;
    Direction: Double;
    FallSpeed: Double;
    Speed: Double;
    Timer: Integer;
    IsRocket: Boolean;
    Exploded: Boolean;
    procedure Init; virtual;
    destructor Destroy; virtual;
    function Frame: Boolean; virtual;
    procedure Explode; virtual;
    procedure Draw( Bmp: PBitmap ); virtual;
    procedure DrawAtPos( Bmp: PBitmap; X, Y: Integer );
  end;

  TExplodeStyle = ( esRadial, esSphere, esMultiHead, esNo );

  PRocket = ^TRocket;
  TRocket = object( TFlare )
  protected
    Color2: TColor;
    CountBeforeExplode: Integer;
    Trace: PList;
    Style: TExplodeStyle;
    procedure Init; virtual;
  public
    destructor Destroy; virtual;
    function Frame: Boolean; virtual;
    procedure Explode; virtual;
    procedure Draw( Bmp: PBitmap ); virtual;
    procedure DrawLine( Bmp: PBitmap; const P1, P2: TPoint; N, WidthLine: Integer );
    procedure CreateFlaresSphere( Distance: Double; FlaresSpeed: Double );
    procedure CreateRocketsSphere( FlaresSpeed: Double );
    procedure CreateFlaresRandom( Distance: Double; FlaresSpeed: Double );
  end;

function NewSalut( Awidth, Aheight, Afrequency, Acount: Integer ): PSalut;
begin
  new( Result, Create );
  Result.Width := Awidth;
  Result.Height := Aheight;
  Result.Frequency := Afrequency;
  Result.ValleyCount := Acount;
end;

procedure DrawPixel( Bmp: PBitmap; X, Y: Integer; C: TRGBQuad );
var Dst: PDWORD;
begin
  if not PtInRect( Bmp.BoundsRect, MakePoint( X, Y ) ) then Exit;
  Dst := Pointer( Integer( Bmp.ScanLine[ Y ] ) + X * 4 );
  Dst^ := Dst^ or DWORD( C );
end;

{ TSalut }

function TSalut.BoundsRect: TRect;
begin
  Result := MakeRect( 0, 0, Width, Height );
end;

function TSalut.CreateRocket: PObj;
var R: PRocket;
begin
  new( R, Create );
  R.Salut := @ Self;
  R.StartPos := MakePoint( Random( Width ), Height );
  R.Direction := ( 45 + Random( 90 ) ) / 180 * PI;
  R.Speed := 20 + Random( 10 ) * 2;
  R.FallSpeed := Random( 10 ) / 5;
  R.CountBeforeExplode := Round( Height * 4 div 5 / R.Speed );
  R.Style := TExplodeStyle( Random( Integer( High( TExplodeStyle ) ) ) );
  R.Color := Colors[ Random( High( Colors ) + 1 ) ];
  R.Color2 := R.Color;
  if Random( 3 ) = 0 then
    R.Color2 := Colors[ Random( High( Colors ) + 1 ) ];
  Result := R;
  EnterCriticalSection( fCritSec );
  TRY
    Rockets.Add( R );
  FINALLY
    LeaveCriticalSection( fCritSec );
  END;
  Sound( seRocket, 20, (R.StartPos.X - Width div 2) * 70 div Width );
end;

destructor TSalut.Destroy;
var i: Integer;
begin
  ShortTimer.Free;
  TmpBmp.Free;
  EnterCriticalSection( fCritSec );
  TRY
    for i := Rockets.Count-1 downto 0 do
      PObj( Rockets.Items[ i ] ).Free;
    for i := Flares.Count-1 downto 0 do
      PObj( Flares.Items[ i ] ).Free;
    Rockets.Free;
    Flares.Free;
  FINALLY
    LeaveCriticalSection( fCritSec );
  END;
  DeleteCriticalSection( fCritSec );
  inherited;
end;

procedure TSalut.Draw2Bmp(Bmp, Mask: PBitmap);
var i: Integer;
    R: PRocket;
    F: PFlare;
begin
  Bmp.PixelFormat := pf32bit;
  EnterCriticalSection( fCritSec );
  TRY
    for i := 0 to Rockets.Count-1 do
    begin
      R := Rockets.Items[ i ];
      R.Draw( Bmp );
    end;
    for i := Flares.Count-1 downto 0 do
    begin
      F := Flares.Items[ i ];
      TRY
      F.Draw( Bmp );
      EXCEPT
      asm
        int 3
      end;
      END;
    end;
  FINALLY
    LeaveCriticalSection( fCritSec );
  END;
end;

procedure TSalut.Draw2DC(DC: HDC; x0, y0: Integer);
var y, x: Integer;
    Dst: PDWORD;
    C: DWORD;
    SaveSeed: Integer;
begin
  if TmpBmp = nil then
    TmpBmp := NewDiBBitmap( Width, Height, pf32bit );
  if TmpFrameIdx <> FrameIdx then
  begin
    SaveSeed := RandSeed;
    RandSeed := 1000;
    TRY
      TmpFrameIdx := FrameIdx;
      for y := 0 to TmpBmp.Height-1 do
      begin
        Dst := TmpBmp.ScanLine[ y ];
        for x := 0 to TmpBmp.Width-1 do
        begin
          C := Round( 255 * sin( y / TmpBmp.Height ) ) + Random( 16 );
          if C > 255 then C := 255;
          Dst^ := C;
          Inc( Dst );
        end;
      end;
    FINALLY
      RandSeed := SaveSeed;
    END;
    {TmpBmp.Canvas.Brush.Color := clNavy; // clBlack;
    TmpBmp.Canvas.FillRect( TmpBmp.BoundsRect );}
    Draw2Bmp( TmpBmp, nil );
  end;
  TmpBmp.Draw( DC, x0, y0 );
end;

procedure TSalut.Frame;
var SaveRandSeed: Integer;
    i, T: Integer;
    R: PRocket;
    F: PFlare;
begin
  SaveRandSeed := RandSeed;
  EnterCriticalSection( fCritSec );
  TRY
    RandSeed := Seed;
    Inc( FrameIdx );
    if ValleyCount > 0 then
    if Rockets.Count = 0 then
    begin
      Inc( TimesNoRockets );
      if TimesNoRockets > Random( 100 div Frequency + 1 ) then
      begin
        TimesNoRockets := 0;
        CreateRocket;
        Dec( ValleyCount );
      end;
    end
      else
    if Rockets.Count < Frequency * 2 then
    begin
      T := 50;
      for i := 0 to Rockets.Count-1 do
      begin
        R := Rockets.Items[ i ];
        if T > R.Timer then
          T := R.Timer;
      end;
      if T > Random( 50 div Frequency + 1 ) then
      begin
        CreateRocket;
        Dec( ValleyCount );
      end;
    end;
    for i := Rockets.Count-1 downto 0 do
    begin
      R := Rockets.Items[ i ];
      R.Frame;
    end;
    for i := Flares.Count-1 downto 0 do
    begin
      F := Flares.Items[ i ];
      F.Frame;
    end;
    if (ValleyCount = 0) and (Rockets.Count = 0) and (Flares.Count = 0) then
      if Assigned( OnEndSalut ) then
        OnEndSalut( @ Self );
  FINALLY
    Seed := RandSeed;
    RandSeed := SaveRandSeed;
    LeaveCriticalSection( fCritSec );
  END;
end;

procedure TSalut.Init;
begin
  inherited;
  Rockets := NewList;
  Flares := NewList;
  Seed := RandSeed;
  if RocketSounds = nil then
  begin
    LoadSounds( RocketSounds, 'Rocket' );
    LoadSounds( ExplodeSounds, 'Explode' );
    LoadSounds( ShortSounds, 'Short' );
  end;
  ShortTimer := NewMMTimer( 173 );
  ShortTimer.OnTimer := ShortTimerTick;
  InitializeCriticalSection( fCritSec );
end;

procedure TSalut.LoadSounds( var List: PList; const Path: String );
var DL: PDirList;
    i, sz: Integer;
    Mem: Pointer;
begin
  List := NewList;
  DL := NewDirListEx( GetStartDir + 'Sounds\Salut\' + Path + '\',
     '*.wav', FILE_ATTRIBUTE_NORMAL );
  TRY
    for i := 0 to DL.Count-1 do
    begin
      sz := DL.Items[ i ].nFileSizeLow;
      if sz = 0 then continue;
      GetMem( Mem, sz );
      if File2Mem( PChar( DL.Path + DL.Names[ i ] ), Mem, sz ) > 0 then
        List.Add( Mem )
      else
        FreeMem( Mem );
    end;
  FINALLY
    DL.Free;
  END;
end;

procedure TSalut.ShortTimerTick(Sender: PObj);
begin
  Dec( ShortTimerCount );
  if ShortTimerCount <= 0 then
    ShortTimer.Enabled := FALSE;
  Sound( seShort, 70, 0 );
end;

procedure TSalut.Sound(Effect: TSoundEffect; Volume,Pan: Integer);
var i: Integer;
    L: PList;
begin
  CASE Effect OF
  seRocket: L := RocketSounds;
  seExplode: L := ExplodeSounds;
  seShort: L := ShortSounds;
  else Exit;
  END;
  if L.Count = 0 then Exit;
  i := Random( L.Count );
  {$IFDEF USEDIRECTSND}
  if MultiWavObj = nil then
    new( MultiWavObj, Create );
  MultiWavObj.Prty := HIGH_PRIORITY_CLASS;
  MultiWavObj.PlayWaveMemory( L.Items[ i ], Volume * 3 div 5, Pan );
  {$ELSE}
  PlaySound( L.Items[ i ], 0, SND_ASYNC or SND_MEMORY or SND_NODEFAULT
             //or SND_NOSTOP //or SND_NOWAIT
             );
  {$ENDIF}
end;

{ TRocket }

procedure TRocket.CreateFlaresRandom(Distance, FlaresSpeed: Double);
var fi, psi: Double;
    F: PFlare;
    X1, Z1, L, X, Y, sinD, cosD, D: Double;
    Fall: Double;
    i: Integer;
begin
  sinD := sin( Direction + PI / 2 );
  cosD := cos( Direction + PI / 2 );
  Fall := Random * 10;
  EnterCriticalSection( Salut.fCritSec );
  TRY
    for i := 1 to 50 + Random(200) do
    begin
      fi := PI * Random;
      psi := PI * 2 * Random;
      new( F, Create );
      Salut.Flares.Add( F );
      F.Salut := Salut;
      F.Color := Color;
      if Random( 2 ) = 1 then
        F.Color := Color2;
      X1 := sin( fi ) * cos( psi );
      //Y1 := sin( fi ) * sin( psi );
      Z1 := cos( fi );

      X := cosD * X1 + sinD * Z1;
      Y := -sinD * X1 + cosD * Z1;
      L := sqrt( sqr( X ) + sqr( Y ) );
      F.Speed := FlaresSpeed * 1.5 * L;
      D := Distance / 2 + Distance / 2 * Random;
      F.StartPos.X := Round( Position.X + D * X );
      F.StartPos.Y := Round( Position.Y + D * Y );
      F.Direction := ArcTan2( -Y, X );
      F.FallSpeed := Fall;
    end;
  FINALLY
    LeaveCriticalSection( Salut.fCritSec );
  END;
end;

procedure TRocket.CreateFlaresSphere(Distance, FlaresSpeed: Double);
var fi, psi: Double;
    F: PFlare;
    X1, Z1, L, X, Y, sinD, cosD: Double;
    Fall: Double;
    Step, M: Integer;
begin
  if Random( 3 ) > 0 then
  begin
    CreateFlaresRandom( Distance, FlaresSpeed );
    Exit;
  end;
  Fall := Random * 10 + 0.1;
  fi := 0;
  Step := 10 + Random( 10 );
  sinD := sin( Direction + PI / 2 );
  cosD := cos( Direction + PI / 2 );
  EnterCriticalSection( Salut.fCritSec );
  TRY
    while fi < PI do
    begin
      psi := 0;
      if fi <= PI / 2 then
        M := Round( (PI/2 - fi) / PI * Step )
      else
        M := Round( (fi - PI/2) / PI * Step );
      while psi < PI * 2 do
      begin
        new( F, Create );
        Salut.Flares.Add( F );
        F.Salut := Salut;
        F.Color := Color;
        if Random( 2 ) = 1 then
          F.Color := Color2;
        X1 := sin( fi ) * cos( psi );
        //Y1 := sin( fi ) * sin( psi );
        Z1 := cos( fi );

        X := cosD * X1 + sinD * Z1;
        Y := -sinD * X1 + cosD * Z1;
        //Z := -sinD * X1 + cosD * Y1;

        L := sqrt( sqr( X ) + sqr( Y ) );

        F.Speed := FlaresSpeed * 1.5 * L;
        F.StartPos.X := Round( Position.X + Distance * X );
        F.StartPos.Y := Round( Position.Y + Distance * Y );
        F.Direction := ArcTan2( -Y, X );

        F.Speed := F.Speed + cos( F.Direction - Direction ) * Speed / 5;

        F.FallSpeed := Fall;
        psi := psi + PI / (Step - M);
      end;
      fi := fi + PI / Step;
    end;
  FINALLY
    LeaveCriticalSection( Salut.fCritSec );
  END;
end;

procedure TRocket.CreateRocketsSphere(FlaresSpeed: Double);
var fi, psi: Double;
    F: PRocket;
    X1, Z1, L, X, Y, sinD, cosD: Double;
    Fall: Double;
    i, Cnt: Integer;
begin
  Fall := Random * 4 + 0.1;
  sinD := sin( Direction + PI / 2 );
  cosD := cos( Direction + PI / 2 );
  Cnt := 5 + Round( 20 );
  //Exit;
  EnterCriticalSection( Salut.fCritSec );
  TRY

    for i := 1 to 20 + Random(80) do
    begin
      fi := PI * Random;
      psi := PI * 2 * Random;

      new( F, Create );
      Salut.Flares.Add( F );
      F.Salut := Salut;
      F.Color := Color;
      if Random( 2 ) = 1 then
        F.Color := Color2;
      X1 := sin( fi ) * cos( psi );
      //Y1 := sin( fi ) * sin( psi );
      Z1 := cos( fi );

      X := cosD * X1 + sinD * Z1;
      Y := -sinD * X1 + cosD * Z1;
      //Z := -sinD * X1 + cosD * Y1;

      L := sqrt( sqr( X ) + sqr( Y ) );

      F.Speed := FlaresSpeed * L;
      F.StartPos.X := Round( Position.X );
      F.StartPos.Y := Round( Position.Y );
      F.Direction := ArcTan2( -Y, X );

      //F.Speed := F.Speed + cos( F.Direction - Direction ) * Speed / 5;

      F.FallSpeed := Fall;
      F.Style := esNo;
      F.CountBeforeExplode := Cnt;
    end;
  FINALLY
    LeaveCriticalSection( Salut.fCritSec );
  END;

end;

destructor TRocket.Destroy;
begin
  Trace.Free;
  inherited;
end;

procedure TRocket.Draw(Bmp: PBitmap);
var i, N: Integer;
    Pos1, Pos2: TPoint;
    WasBright: Integer;
begin
  WasBright := Bright;
  if Style <> esNo then
    Bright := MaxBright;
  N := 20;
  //inherited;
  for i := Trace.Count-2 downto 0 do
  begin
    Pos1.X := SmallInt( LoWord( Integer( Trace.Items[ i+1 ] ) ) );
    Pos1.Y := SmallInt( HiWord( Integer( Trace.Items[ i+1 ] ) ) );
    Pos2.X := SmallInt( LoWord( Integer( Trace.Items[ i ] ) ) );
    Pos2.Y := SmallInt( HiWord( Integer( Trace.Items[ i ] ) ) );
    DrawLine( Bmp, Pos1, Pos2, N, i+1 );
    Bright := Bright * 5 div 6;
    N := N * 2 div 3;
  end;
  if Style = esNo then
    Bright := WasBright;
end;

procedure TRocket.DrawLine(Bmp: PBitmap; const P1, P2: TPoint; N: Integer; WidthLine: Integer);
var k, offset, angle: Double;
    sinA, cosA: Double;
    P: TPoint;
begin
  while N > 0 do
  begin
    k := Random;
    offset := Random * WidthLine / 2;
    angle := random * PI * 2;
    sinA := sin( angle );
    cosA := cos( angle );
    P.X := Round( P1.X + (P2.X - P1.X) * k + offset * cosA );
    P.Y := Round( P1.Y + (P2.Y - P1.Y) * k + offset * sinA );
    DrawAtPos( Bmp, P.X, P.Y );
    Dec( N );
  end;
end;

procedure TRocket.Explode;
var i: Integer;
    R: PRocket;
begin
  CASE Style OF
  esRadial:
    begin
      Salut.Sound( seExplode, 100, (Position.X - Salut.Width div 2) * 100 div Salut.Width );
      if Random( 4 ) <> 0 then
        CreateRocketsSphere( 4 + Random( 4 ) )
      else
        CreateFlaresSphere( 0, 4 + Random( 4 ) );
    end;
  esSphere:
    begin
      Salut.ShortTimerCount := Max( Salut.ShortTimerCount, 2 + Random( 4 ) );
      Salut.ShortTimer.Enabled := TRUE;
      CreateFlaresSphere( 20 + Random( 20 ), 3 );
    end;
  esMultiHead:
    for i := 0 to Random( 5 ) do
    begin
      R := PRocket( Salut.CreateRocket );
      R.StartPos := Position;
      R.Direction := Random * PI * 2;
      R.CountBeforeExplode := R.CountBeforeExplode div 2;
      R.Speed := R.Speed / 2;
      R.Style := TExplodeStyle( Random( 2 ) );
    end;
  END;
  Exploded := TRUE;
end;

function TRocket.Frame: Boolean;
begin
  if not Exploded then
  begin
    if Style <> esNo then
      Bright := MaxBright;
    Result := inherited Frame;
    if Result then Exit;
    if Timer >= CountBeforeExplode then
      Explode
    else
    begin
      Trace.Add( Pointer( Position.X or (Position.Y shl 16) ) );
      if Trace.Count > 4 then
        Trace.Delete( 0 );
    end;
  end
    else
  if Style <> esNo then
  begin
    if Trace.Count > 0 then
      Trace.Delete( 0 );
  end
    else
    begin
      Result := inherited Frame;
      if Result then Exit;
      if Timer >= CountBeforeExplode + 5 then
      begin
        Result := TRUE;
        Free;
        Exit;
      end;
    end;
  Result := FALSE;
  if Trace.Count = 0 then
  begin
    Result := TRUE;
    Free;
  end;
end;

procedure TRocket.Init;
begin
  inherited;
  Trace := NewList;
  IsRocket := TRUE;
end;

{ TFlare }

destructor TFlare.Destroy;
begin
  EnterCriticalSection( Salut.fCritSec );
  TRY
    Salut.Flares.Remove( @ Self );
    Salut.Rockets.Remove( @ Self );
  FINALLY
    LeaveCriticalSection( Salut.fCritSec );
  END;
  inherited;
end;

procedure TFlare.Draw(Bmp: PBitmap);
begin
  DrawAtPos( bmp, Position.X, Position.Y );
end;

procedure TFlare.DrawAtPos(Bmp: PBitmap; X, Y: Integer);
var C: TRGBQuad;
begin
  C := Color2RGBQuad( Color );
  C.rgbBlue := C.rgbBlue * Bright div MaxBright;
  C.rgbGreen := C.rgbGreen * Bright div MaxBright;
  C.rgbRed := C.rgbRed * Bright div MaxBright;
  DrawPixel( Bmp, X, Y, C );
  if IsRocket then
  begin
    C.rgbBlue := C.rgbBlue shr 1;
    C.rgbGreen := C.rgbGreen shr 1;
    C.rgbRed := C.rgbRed shr 1;
  end;
  DrawPixel( Bmp, X-1, Y, C );
  DrawPixel( Bmp, X+1, Y, C );
  DrawPixel( Bmp, X, Y-1, C );
  DrawPixel( Bmp, X, Y+1, C );
  if IsRocket then
  begin
    C.rgbBlue := C.rgbBlue shr 1;
    C.rgbGreen := C.rgbGreen shr 1;
    C.rgbRed := C.rgbRed shr 1;
  end
    else
  begin
    C.rgbBlue := C.rgbBlue * 2 div 3;
    C.rgbGreen := C.rgbGreen * 2 div 3;
    C.rgbRed := C.rgbRed * 2 div 3;
  end;
  DrawPixel( Bmp, X-1, Y-1, C );
  DrawPixel( Bmp, X+1, Y+1, C );
  DrawPixel( Bmp, X+1, Y-1, C );
  DrawPixel( Bmp, X-1, Y+1, C );
  if not IsRocket then
  begin
    C.rgbBlue := C.rgbBlue * 2 div 3;
    C.rgbGreen := C.rgbGreen * 2 div 3;
    C.rgbRed := C.rgbRed * 2 div 3;
    DrawPixel( Bmp, X-1, Y-2, C );
    DrawPixel( Bmp, X, Y-2, C );
    DrawPixel( Bmp, X+1, Y-2, C );
    DrawPixel( Bmp, X-1, Y+2, C );
    DrawPixel( Bmp, X, Y+2, C );
    DrawPixel( Bmp, X+1, Y+2, C );
    DrawPixel( Bmp, X-2, Y-1, C );
    DrawPixel( Bmp, X-2, Y, C );
    DrawPixel( Bmp, X-2, Y+1, C );
    DrawPixel( Bmp, X+2, Y-1, C );
    DrawPixel( Bmp, X+2, Y, C );
    DrawPixel( Bmp, X+2, Y+1, C );
  end;
end;

procedure TFlare.Explode;
begin
  Free;
end;

function TFlare.Frame: Boolean;
var X1, Y1: Double;
    sinA, cosA: Double;
    t: Integer;
begin
  Inc( Timer );
  if not IsRocket and (Timer >= 10) or
     IsRocket and Exploded then
  begin
    Bright := Bright * 2 div 3;
    if Bright = 0 then
    begin
      Result := TRUE;
      Free;
      Exit;
    end;
  end;
  X1 := StartPos.X;
  Y1 := StartPos.Y;
  sinA := sin( Direction );
  cosA := cos( Direction );
  for t := 1 to Timer do
  begin
    X1 := X1 + Speed * cosA;
    Y1 := Y1 - Speed * sinA + FallSpeed;
  end;
  Position.X := Round( X1 );
  Position.Y := Round( Y1 );
  if not PtInRect( Salut.BoundsRect, Position ) then
  begin
    Result := TRUE;
    Free;
    Exit;
  end;
  Result := FALSE;
  FallSpeed := FallSpeed * 1.03;
end;

procedure TFlare.Init;
begin
  inherited;
  Bright := MaxBright;
end;

initialization
finalization
  RocketSounds.Release;
  ExplodeSounds.Release;
  ShortSounds.Release;

end.
